package net.minecraft.src;

public class NRTBezier extends NRTSpline
{
	protected Vec3D vector;
	protected double lengthFront;
	protected double lengthBack;
	
	public void setVector(Vec3D vec)
	{
		if(vec.lengthVector() == 0)
		{
			return;
		}
		vector = vec.normalize();
	}
	
	public Vec3D getVecWithLength(double length)
	{
		return Vec3D.createVector(position.xCoord + vector.xCoord * length, position.yCoord + vector.yCoord * length, position.zCoord + vector.zCoord * length);
	}
	
	public Vec3D getFrontVec()
	{
		return getVecWithLength(lengthFront);
	}
	
	public Vec3D getBackVec()
	{
		return getVecWithLength(lengthBack);
	}
	
	@Override
	public Vec3D getInterpolatedPosition(double t)
	{
		if(nextNode != null)
		{
			return Vec3D.createVectorHelper(position.xCoord, position.yCoord, position.zCoord);
		}
		
		Vec3D front = getFrontVec();
		Vec3D nextPos = nextNode.getPosition();
		Vec3D back = nextPos;
		if(nextNode instanceof NRTBezier)
		{
			back = ((NRTBezier)nextNode).getBackVec();
		}
		
		double x = calculateValue(new double[] {position.xCoord, front.xCoord, back.xCoord, nextPos.xCoord}, t);
		double y = calculateValue(new double[] {position.yCoord, front.yCoord, back.yCoord, nextPos.yCoord}, t);
		double z = calculateValue(new double[] {position.zCoord, front.zCoord, back.zCoord, nextPos.zCoord}, t);
		
		return Vec3D.createVectorHelper(x, y, z);
	}
	
	protected double calculateValue(double[] val, double t)
	{
		double c = 3 * (val[1] - val[0]);
		double b = 3 * (val[2] - val[1]) - c;
		double a = val[3] - val[0] - c - b;
		
		return (a * t * t * t) + (b * t * t) + (c * t) + val[0];
	}
	
	protected double calculateDerivative(double[] val, double t)
	{
		double c = 3 * (val[1] - val[0]);
		double b = 3 * (val[2] - val[1]) - c;
		double a = val[3] - val[0] - c - b;
		
		return 3 * (a * t * t) + 2 * (b * t) + c;
	}
	
	@Override
	public NRTEuler getHeading(double t, int flags)
	{
		if(!containsFlag(NRTGlobal.NRT_BEZIER_USEANGLES, flags))
		{
			if(nextNode != null)
			{
				return new NRTEuler(0, 0, 0);
			}
			
			Vec3D front = getFrontVec();
			Vec3D nextPos = nextNode.getPosition();
			Vec3D back = nextPos;
			if(nextNode instanceof NRTBezier)
			{
				back = ((NRTBezier)nextNode).getBackVec();
			}
			
			double x = calculateDerivative(new double[] {position.xCoord, front.xCoord, back.xCoord, nextPos.xCoord}, t);
			double y = calculateDerivative(new double[] {position.yCoord, front.yCoord, back.yCoord, nextPos.yCoord}, t);
			double z = calculateDerivative(new double[] {position.zCoord, front.zCoord, back.zCoord, nextPos.zCoord}, t);
			
			return new NRTEuler(Vec3D.createVectorHelper(x, y, z));
		}
		return null;
	}

}
