package net.minecraft.src;

import java.util.ArrayList;

public class NRTBone
{
	protected ArrayList<NRTBone> child;
	protected NRTBone parent;
	protected NRTQuaternion baseQuat;
	protected NRTQuaternion relQuat;
	protected Vec3D baseVec;
	
	public NRTBone()
	{
		this(null);
	}
	
	public NRTBone(NRTBone parentNode)
	{
		parent = parentNode;
		if(parentNode != null)
		{
			parent.addChild(this);
		}
		baseQuat = new NRTQuaternion(1, 0, 0, 0);
		baseVec = Vec3D.createVectorHelper(1, 0, 0);
	}
	
	public void addChild(NRTBone childNode)
	{
		child.add(childNode);
	}

	public void setBaseRotation(Vec3D vec, float angle)
	{
		baseQuat.setQuaternion(vec, angle);
	}
	
	public void setBaseRotation(float pitch, float yaw, float roll)
	{
		baseQuat.setQuaternion(pitch, yaw, roll);
	}
	
	public void setBaseRotation(float w, float x, float y, float z)
	{
		baseQuat.setQuaternion(w, x, y, z);
	}
	
	public void setRotation(Vec3D vec, float angle)
	{
		relQuat.setQuaternion(vec, angle);
	}
	
	public void setRotation(float pitch, float yaw, float roll)
	{
		relQuat.setQuaternion(pitch, yaw, roll);
	}
	
	public void setRotation(float w, float x, float y, float z)
	{
		relQuat.setQuaternion(w, x, y, z);
	}
	
	public void setBaseVector(double x, double y, double z)
	{
		baseVec.xCoord = x;
		baseVec.yCoord = y;
		baseVec.zCoord = z;
	}
	
	public NRTQuaternion getRelativeRotation()
	{
		return relQuat;
	}
	
	public NRTQuaternion getTotalRelativeRotation()
	{
		if(parent != null)
		{
			return relQuat.getMul(parent.getTotalRelativeRotation());
		}
		return new NRTQuaternion(relQuat);
	}
	
	public NRTQuaternion getAbsoluteRotation()
	{
		if(parent != null)
		{
			return relQuat.getMul(baseQuat.getMul(parent.getAbsoluteRotation()));
		}
		return relQuat.getMul(baseQuat);
	}
	
	public double getLength()
	{
		return baseVec.lengthVector();
	}
	
	public Vec3D getRotatedVector()
	{
		return getTotalRelativeRotation().rotateVector(baseVec);
	}
	
	public Vec3D getTotalRotatedVector()
	{
		if(parent != null)
		{
			Vec3D vec = parent.getTotalRotatedVector();
			Vec3D vec2 = getRotatedVector();
			vec.addVector(vec2.xCoord, vec2.yCoord, vec2.zCoord);
			return vec;
		}
		return getRotatedVector();
	}
}
